from typing import Optional
import flask
from gammu import EncodeSMS  # pylint: disable=no-name-in-module)
from gammu.smsd import SMSD


class GammuFlask:
    name = 'gammu-flask'
    app = None
    smsd = None

    def __init__(self, app: Optional[flask.Flask] = None):
        if app is not None:
            self.init_app(app)

    def init_app(self, app: flask.Flask) -> None:
        if not hasattr(app, 'extensions'):
            app.extensions = {}
        if self.name in app.extensions:
            raise ValueError('Already registered extension {}.'.format(self.name))
        app.extensions[self.name] = self

        self.app = app

        self.smsd = SMSD(self.app.config.get('GAMMU_CONFIG_PATH'))

    def get_status(self) -> dict:
        if not self.smsd:
            raise Exception('GammuFlask is not initiated!')
        return self.smsd.GetStatus()

    def inject_sms(self, number: str, sms_info: dict, smsc: Optional[dict] = None) -> str:
        """
        Inject SMS to SMSD queue
        :param number: string Phone number to send
        :param sms_info: dict of type SMS Info Object
        :param smsc: Optional dict SMSC Object
        :return:
        """
        if not self.smsd:
            raise Exception('GammuFlask is not initiated!')

        encoded = EncodeSMS(sms_info)
        messages_to_send = []
        for message in encoded:  # message is of type SMS Object
            message['SMSC'] = smsc or {'Location': 1}  # SMSC Object
            message['Number'] = number
            messages_to_send.append(message)

        return self.smsd.InjectSMS(messages_to_send)
